@@ -279,6 +279,11 @@ def get_conversion_factor(input_unit, output_unit, filter_name="WFI"):
     conversion: astropy unit conversion
     """
 
+    if filter_name not in dict_equivalencies:
+        upipe.print_warning("Didn't find conversion for this filter. "
+                            "Using 1.0 as a conversion factor")
+        return 1.0
+
     equivalencies = dict_equivalencies[filter_name]
     # First testing if the quantities are Quantity
     # If not, transform them 
@@ -947,11 +952,12 @@ class AlignMusePointing(object):
             upipe.print_info("              {0:8.4f} {1:8.4f}".format(
                     self.total_off_arcsec[nima][0],
                     self.total_off_arcsec[nima][1]))
-        upipe.print_info("Total in PIXEL")
+        upipe.print_info("Total in PIXEL     X        Y         ROT(DEG) ")
         for nima in range(self.nimages):
-            upipe.print_info("              {0:8.4f} {1:8.4f}".format(
+            upipe.print_info("              {0:8.4f} {1:8.4f} {2:8.4f}".format(
                     self.total_off_pixel[nima][0],
-                    self.total_off_pixel[nima][1]))
+                    self.total_off_pixel[nima][1],
+                    self.muse_rotangles[nima]))
 
     def save_fits_offset_table(self, name_output_table=None, 
             folder_output_table=None,
@@ -1115,8 +1121,7 @@ class AlignMusePointing(object):
         self.shift_arcsecond(extra_arcsec, nima)
 
         # Compare contours if plot is set to True
-        if self.plot:
-            self.compare(nima=nima, **kwargs)
+        self.compare(nima=nima, **kwargs)
 
     def _get_list_muse_images(self):
         """Extract the name of the muse images
@@ -1658,10 +1663,13 @@ class AlignMusePointing(object):
             print("Low / High level REF  flux: "
                     "{0:8.4e} {1:8.4e}".format(lowlevel_ref, highlevel_ref))
 
-        # Save the frames if in debug mode
-        if self._debug:
-            self._temp_refdata = refdata
-            self._temp_musedata = musedata
+        # Save the frames in case this is needed
+        self._temp_refdata = refdata
+        self._temp_musedata = musedata
+
+        # Stop here if plot is not needed
+        if not self.plot:
+            return
 
         # Get the WCS from mpdaf to allow rotation if needed
         refwcs = self.list_wcs_proj_refhdu[nima]
@@ -1670,7 +1678,7 @@ class AlignMusePointing(object):
         plotwcs = awcs.WCS(self.list_offmuse_hdu[nima].header)
 
         # Apply rotation in degrees
-        # Apply it to the rerence image not to couple it with the offset
+        # Apply it to the reference image not to couple it with the offset
         if rotation != 0:
             if self.verbose:
                 upipe.print_warning("Apply a rotation of "
@@ -1741,8 +1749,9 @@ class AlignMusePointing(object):
                       + highlevel_muse)
             chunk_x = musedata.shape[0] // (ncuts + 1)
             chunk_y = musedata.shape[1] // (ncuts + 1)
-            ax.plot(diffima[np.arange(ncuts)*chunk_x,:].T, 'k-')
-            ax.plot(diffima[:,np.arange(ncuts)*chunk_x], 'r-')
+            ax.plot(diffima[np.arange(ncuts)*chunk_x,:].T, 'k-', label='X')
+            ax.plot(diffima[:,np.arange(ncuts)*chunk_y], 'r-', label='Y')
+            ax.legend(loc=0)
             ax.set_ylim(-20,20)
             ax.set_xlabel("[pixels]", fontsize=20)
             ax.set_ylabel("[%]", fontsize=20)
