@@ -37,9 +37,10 @@ from pymusepipe.create_sof import SofPipe
 from pymusepipe.init_musepipe import InitMuseParameters
 import pymusepipe.util_pipe as upipe
 from pymusepipe import musepipe, prep_recipes_pipe
-from pymusepipe.config_pipe import default_filter_list, default_PHANGS_filter_list, dic_combined_folders
-from pymusepipe.config_pipe import default_prefix_wcs, default_prefix_mask, default_prefix_wcs_mosaic
-from pymusepipe.config_pipe import dic_listObject, lambdaminmax_for_wcs, lambdaminmax_for_mosaic
+from pymusepipe.config_pipe import (default_filter_list, default_PHANGS_filter_list,
+                                    dic_combined_folders, default_prefix_wcs,
+                                    default_prefix_mask, prefix_mosaic, dic_listObject,
+                                    lambdaminmax_for_wcs, lambdaminmax_for_mosaic)
 from pymusepipe.mpdaf_pipe import MuseCube
 
 # Default keywords for MJD and DATE
@@ -243,11 +244,12 @@ def get_pixtable_list(target_path="", list_pointings=None, suffix=""):
 class MusePointings(SofPipe, PipeRecipes):
     def __init__(self, targetname=None, list_pointings="all",
                  dic_exposures_in_pointings=None,
-                 suffix_fixed_pixtables="tmask",
+                 prefix_fixed_pixtables="tmask",
                  folder_config="",
                  rc_filename=None, cal_filename=None,
                  combined_folder_name="Combined", suffix="",
                  offset_table_name=None,
+                 folder_offset_table=None,
                  log_filename="MusePipeCombine.log",
                  verbose=True, debug=False, **kwargs):
         """Initialisation of class muse_expo
@@ -267,10 +269,10 @@ class MusePointings(SofPipe, PipeRecipes):
             Default is False
         vsystemic: float 
             Default is 0. Systemic velocity of the galaxy [in km/s]
-        suffix_fixed_pixtables: str
+        prefix_fixed_pixtables: str
             Suffix for fixed PixTables. Default is ''.
         use_fixed_pixtables: bool
-            Default is False. If True, will use suffix_fixed_pixtables to filter out
+            Default is False. If True, will use prefix_fixed_pixtables to filter out
             Pixtables which have been fixed.
 
         Other possible entries
@@ -302,7 +304,7 @@ class MusePointings(SofPipe, PipeRecipes):
 
         # Including or not the fixed Pixtables in place of the original ones
         self.use_fixed_pixtables = kwargs.pop("use_fixed_pixtables", False)
-        self.suffix_fixed_pixtables = suffix_fixed_pixtables
+        self.prefix_fixed_pixtables = prefix_fixed_pixtables
 
         # Setting other default attributes
         if log_filename is None:
@@ -310,7 +312,7 @@ class MusePointings(SofPipe, PipeRecipes):
             upipe.print_info("The Log file will be {0}".format(log_filename))
         self.log_filename = log_filename
         self.suffix = suffix
-        folder_offset_table = kwargs.pop("folder_offset_table", None)
+        self.add_targetname = kwargs.pop("add_targetname", True)
 
         # End of parameter settings #########################
 
@@ -364,7 +366,8 @@ class MusePointings(SofPipe, PipeRecipes):
             upipe.safely_create_folder(self._dic_combined_folders[folder], verbose=verbose)
 
         # Checking input pointings and pixtables
-        self._check_pointings(dic_exposures_in_pointings)
+        self._pixtab_in_comb_folder = kwargs.pop("pixtab_in_comb_folder", True)
+        self._get_pixtable_list(dic_exposures_in_pointings)
 
         # Checking input offset table and corresponding pixtables
         self._check_offset_table(offset_table_name, folder_offset_table)
@@ -377,7 +380,7 @@ class MusePointings(SofPipe, PipeRecipes):
         # Going back to initial working directory
         self.goto_origfolder()
 
-    def _add_targetname(self, name):
+    def _add_targetname(self, name, asprefix=True):
         """Add targetname to input name and return it
         
         Input
@@ -389,61 +392,15 @@ class MusePointings(SofPipe, PipeRecipes):
         new name including targetname
         """
         if self.add_targetname:
-            return "{0}_{1}".format(self.targetname, name)
+            if asprefix:
+                return "{0}_{1}".format(self.targetname, name)
+            else:
+                return "{0}{1}_".format(name, self.targetname)
         else:
             return name
 
-    def run_combine_all_single_pointings_withmasks(self, combine=True, masks=True, 
-            mosaic_wcs=True, perpointing_combine=True, **kwargs):
-        """Run all combine recipes including WCS and masks
-
-        combine: bool [True]
-            Default is True. Will run the combine for all pointings.
-
-        masks: bool [True]
-            Will run the combined WCS and the individual pointing ones
-            (and masks).
-
-        mosaic_wcs (bool): [True]. Reference WCS for the full mosaic
-            
-        perpointing_combine: bool [True]
-            Will run individual pointings using the WCS.
-        """
-        lambdaminmax = kwargs.pop("lambdaminmax", lambdaminmax_for_mosaic)
-        if combine:
-            upipe.print_info("Running the mosaic combine")
-            offset_table_name = kwargs.get("offset_table_name", None)
-            folder_offset_table = kwargs.get("folder_offset_table",
-                                             self.folder_offset_table)
-            if offset_table_name is not None:
-                self._check_offset_table(offset_table_name, folder_offset_table)
-            self.run_combine(lambdaminmax=lambdaminmax,
-                             offset_table_name=offset_table_name,
-                             folder_offset_table=folder_offset_table)
-
-        if masks:
-            # Creating the full mosaic WCS first with a narrow lambda range
-            upipe.print_info("Start creating the narrow-lambda WCS and Masks")
-            _ = self.create_combined_wcs()
-            # Then creating the mask WCS for each pointing
-            upipe.print_info("Start creating the individual Pointings Masks")
-            self.create_all_pointings_mask_wcs(lambdaminmax_mosaic=lambdaminmax, 
-                                               **kwargs)
-
-        if mosaic_wcs:
-            # Creating a reference WCS for the Full Mosaic with the right 
-            # Spectral coverage for a full mosaic
-            upipe.print_info("Start creating the full-lambda WCS")
-            self._combined_wcs_name = self.create_combined_wcs(
-                prefix_wcs=default_prefix_wcs_mosaic,
-                lambdaminmax_wcs=lambdaminmax_for_mosaic)
-
-        if perpointing_combine:
-            upipe.print_info("Running the Individual Pointing combine")
-            # Then merging each single pointing using the masks
-            self.run_combine_all_single_pointings(**kwargs)
-
-    def _check_pointings(self, dic_exposures_in_pointings=None):
+    def _get_pixtable_list(self, dic_exposures_in_pointings=None,
+                           folder_pixtables=None):
         """Check if pointings and dictionary are compatible
         """
         # Dictionary of exposures to select per pointing
@@ -451,6 +408,8 @@ class MusePointings(SofPipe, PipeRecipes):
 
         # Getting the pieces of the names to be used for pixtabs
         pixtable_suffix = prep_recipes_pipe.dic_products_scipost['individual'][0]
+        if self._pixtab_in_comb_folder and self.add_targetname:
+            pixtable_suffix = self._add_targetname(pixtable_suffix)
 
         # Initialise the dictionary of pixtabs to be found in each pointing
         self.dic_pixtabs_in_pointings = {}
@@ -458,22 +417,32 @@ class MusePointings(SofPipe, PipeRecipes):
         # Loop on Pointings
         for pointing in self.list_pointings:
             # get the path
-            path_pointing = getattr(self.paths, self.dic_name_pointings[pointing])
+            if self._pixtab_in_comb_folder:
+                path_pixtables = self.paths.cubes
+                pointing_suffix = "_P{0:02d}".format(np.int(pointing))
+            else:
+                path_pointing = getattr(self.paths, self.dic_name_pointings[pointing])
+                path_pixtables = path_pointing + self.pipe_params.object
+                pointing_suffix = ""
             # List existing pixtabs, using the given suffix
-            list_pixtabs = glob.glob(path_pointing + self.pipe_params.object +
-                                     "{0}{1}*fits".format(pixtable_suffix, self.suffix))
+            list_pixtabs = glob.glob(path_pixtables + "{0}{1}{2}*fits".format(
+                                     pixtable_suffix, self.suffix,
+                                     pointing_suffix))
 
             # Take (or not) the fixed pixtables
             if self.use_fixed_pixtables:
-                suffix_to_consider = "{0}{1}".format(self.suffix_fixed_pixtables,
+                prefix_to_consider = "{0}{1}".format(self.prefix_fixed_pixtables,
                                                      pixtable_suffix)
-                list_fixed_pixtabs = glob.glob(path_pointing + self.pipe_params.object +
-                                               "{0}{1}*fits".format(suffix_to_consider, self.suffix))
+                list_fixed_pixtabs = glob.glob(path_pixtables +
+                                               "{0}{1}{2}*fits".format(
+                                                   prefix_to_consider,
+                                                   self.suffix,
+                                                   pointing_suffix))
 
                 # Looping over the existing fixed pixtables
                 for fixed_pixtab in list_fixed_pixtabs:
                     # Finding the name of the original one
-                    orig_pixtab = fixed_pixtab.replace(suffix_to_consider,
+                    orig_pixtab = fixed_pixtab.replace(prefix_to_consider,
                                                        pixtable_suffix)
                     if orig_pixtab in list_pixtabs:
                         # If it exists, remove it
@@ -504,7 +473,8 @@ class MusePointings(SofPipe, PipeRecipes):
                 select_list_pixtabs = []
                 # this is the list of exposures to consider
                 if pointing not in self.dic_exposures_in_pointings:
-                    upipe.print_warning("Pointing {} not in dictionary - skipping".format(pointing))
+                    upipe.print_warning("Pointing {} not in dictionary "
+                                        "- skipping".format(pointing))
                 else:
                     list_expo = self.dic_exposures_in_pointings[pointing]
                     # We loop on that list
@@ -683,6 +653,60 @@ class MusePointings(SofPipe, PipeRecipes):
         for name in self.pipe_params._dic_folders_target:
             setattr(self.paths, name, joinpath(self.paths.target, self.pipe_params._dic_folders_target[name]))
 
+    def create_reference_wcs(self, pointings_wcs=True, mosaic_wcs=True,
+                             reference_cube=True, **kwargs):
+        """Create the WCS reference files, for all individual pointings and for
+        the mosaic.
+
+        pointings_wcs: bool [True]
+            Will run the individual pointing WCS
+        mosaic_wcs: bool [True]
+            Will run the combined WCS
+        lambdaminmax: [float, float]
+
+        """
+        lambdaminmax = kwargs.pop("lambdaminmax", lambdaminmax_for_mosaic)
+
+        # Creating the reference cube if not existing
+        if reference_cube:
+            refcube_name = kwargs.pop("refcube_name", None)
+            if refcube_name is None:
+                upipe.print_info("@@@@@@@ First creating a reference (mosaic) "
+                                 "cube with short spectral range @@@@@@@")
+                self.run_combine(lambdaminmax=lambdaminmax_for_wcs,
+                                 filter_list="white",
+                                 prefix_all=default_prefix_wcs,
+                                 targetname_asprefix=False)
+                wcs_refcube_name = self._combined_cube_name
+            else:
+                upipe.print_info("@@@@@@@@ Start creating the narrow-lambda "
+                                 "WCS and Masks @@@@@@@@")
+                wcs_refcube_name = self.create_combined_wcs(
+                                       refcube_name=refcube_name)
+        else:
+            # getting the name of the final datacube (mosaic)
+            cube_suffix = prep_recipes_pipe.dic_products_scipost['cube'][0]
+            cube_name = "{0}{1}.fits".format(default_prefix_wcs,
+                                          self._add_targetname(cube_suffix))
+            wcs_refcube_name = joinpath(self.paths.cubes, cube_name)
+
+        if pointings_wcs:
+            # Creating the full mosaic WCS first with a narrow lambda range
+            # Then creating the mask WCS for each pointing
+            upipe.print_info("@@@@@@@@ Start creating the individual "
+                             "Pointings Masks @@@@@@@@")
+            self.create_all_pointings_wcs(lambdaminmax_mosaic=lambdaminmax,
+                                          **kwargs)
+
+        if mosaic_wcs:
+            # Creating a reference WCS for the Full Mosaic with the right
+            # Spectral coverage for a full mosaic
+            upipe.print_info("@@@@@@@ Start creating the full-lambda WCS @@@@@@@")
+            self._combined_wcs_name = self.create_combined_wcs(
+                                        prefix_wcs=prefix_mosaic,
+                                        lambdaminmax_wcs=lambdaminmax_for_mosaic,
+                                        refcube_name=wcs_refcube_name)
+
     def run_combine_all_single_pointings(self,
                                          add_suffix="",
                                          sof_filename='pointings_combine',
@@ -766,11 +790,12 @@ class MusePointings(SofPipe, PipeRecipes):
                          ref_wcs=ref_wcs,
                          **kwargs)
 
-    def create_all_pointings_mask_wcs(self, filter_list="white", **kwargs):
+    def create_all_pointings_wcs(self, filter_list="white", **kwargs):
         """Create all pointing masks one by one
         as well as the wcs for each individual pointings. Using the grid
         from the global WCS of the mosaic but restricting it to the 
         range of non-NaN.
+        Hence this needs a global WCS mosaic as a reference to work.
 
         Input
         -----
@@ -782,17 +807,17 @@ class MusePointings(SofPipe, PipeRecipes):
 
         # Additional suffix if needed
         for pointing in list_pointings:
-            upipe.print_info("Making WCS Mask for pointing {0:02d}".format(pointing))
-            _ = self.create_pointing_mask_wcs(pointing=pointing,
-                                          filter_list=filter_list,
-                                          **kwargs)
+            upipe.print_info("Making WCS Mask for "
+                             "pointing {0:02d}".format(pointing))
+            _ = self.create_pointing_wcs(pointing=pointing,
+                                         filter_list=filter_list, **kwargs)
 
-    def create_pointing_mask_wcs(self, pointing, 
+    def create_pointing_wcs(self, pointing,
             lambdaminmax_mosaic=lambdaminmax_for_mosaic,
             filter_list="white", **kwargs):
         """Create the mask of a given pointing
-        And also a WCS file which can then be used to compute individual pointings
-        with a fixed WCS.
+        And also a WCS file which can then be used to compute individual
+        pointings with a fixed WCS.
 
         Input
         -----
@@ -828,13 +853,11 @@ class MusePointings(SofPipe, PipeRecipes):
 
         # Adding targetname for the final names
         prefix_mask = self._add_targetname(prefix_mask)
-        prefix_wcs = self._add_targetname(prefix_wcs)
+        prefix_wcs = self._add_targetname(prefix_wcs, asprefix=False)
 
         name_mask = "{0}DATACUBE_FINAL_P{1:02d}.fits".format(
             prefix_mask, np.int(pointing))
-        finalname_mask = "{0}P{1:02d}.fits".format(prefix_mask,
-                                                   np.int(pointing))
-        finalname_wcs = "{0}P{1:02d}.fits".format(prefix_wcs,
+        finalname_wcs = "{0}DATACUBE_FINAL_P{1:02d}.fits".format(prefix_wcs,
                                                   np.int(pointing))
 
         # First create a subcube without all the Nan
@@ -916,7 +939,7 @@ class MusePointings(SofPipe, PipeRecipes):
         upipe.print_info("...Done")
         return full_cname
 
-    def create_combined_wcs(self, name_cube=None, 
+    def create_combined_wcs(self, refcube_name=None,
             lambdaminmax_wcs=lambdaminmax_for_wcs,
             **kwargs):
         """Create the reference WCS from the full mosaic
@@ -924,7 +947,7 @@ class MusePointings(SofPipe, PipeRecipes):
 
         Input
         -----
-        name_cube: str
+        refcube_name: str
             Name of the cube. Can be None, and then the final
             datacube from the combine folder will be used.
         wave1: float - optional
@@ -940,20 +963,20 @@ class MusePointings(SofPipe, PipeRecipes):
         # Adding targetname in names or not
         self.add_targetname = kwargs.pop("add_targetname", True)
 
-        if name_cube is None:
+        if refcube_name is None:
             # getting the name of the final datacube (mosaic)
             cube_suffix = prep_recipes_pipe.dic_products_scipost['cube'][0]
             cube_suffix = self._add_targetname(cube_suffix)
-            name_cube = joinpath(self.paths.cubes, cube_suffix + ".fits")
+            refcube_name = joinpath(self.paths.cubes, cube_suffix + ".fits")
 
         # test if cube exists
-        if not os.path.isfile(name_cube):
+        if not os.path.isfile(refcube_name):
             upipe.print_error("[combine/create_combined_wcs] File {0} does not exist "
-                              "- Aborting.".format(name_cube))
+                              "- Aborting.".format(refcube_name))
             return
 
         # Opening the cube via MuseCube
-        refcube = MuseCube(filename=name_cube)
+        refcube = MuseCube(filename=refcube_name)
 
         # Creating the new cube
         prefix_wcs = kwargs.pop("prefix_wcs", default_prefix_wcs)
@@ -964,8 +987,8 @@ class MusePointings(SofPipe, PipeRecipes):
 
         # Now transforming this into a bona fide 1 extension WCS file
         combined_wcs_name = joinpath(cfolder, cname)
-        d = pyfits.getdata(full_cname, ext=1)
-        h = pyfits.getheader(full_cname, ext=1)
+        d = pyfits.getdata(combined_wcs_name, ext=1)
+        h = pyfits.getheader(combined_wcs_name, ext=1)
         hdu = pyfits.PrimaryHDU(data=d, header=h)
         hdu.writeto(combined_wcs_name, overwrite=True)
         upipe.print_info("...Done")
@@ -999,10 +1022,11 @@ class MusePointings(SofPipe, PipeRecipes):
         expotype = kwargs.pop("expotype", 'REDUCED')
 
         # Adding target name as prefix or not
-        self.add_targetname = kwargs.pop("add_targetname", True)
+        self.add_targetname = kwargs.pop("add_targetname", self.add_targetname)
+        asprefix = kwargs.pop("targetname_asprefix", True)
         prefix_wcs = kwargs.pop("prefix_wcs", default_prefix_wcs)
         prefix_all = kwargs.pop("prefix_all", "")
-        prefix_all = self._add_targetname(prefix_all)
+        prefix_all = self._add_targetname(prefix_all, asprefix)
 
         if "offset_table_name" in kwargs:
             offset_table_name = kwargs.pop("offset_table_name")
