@@ -35,8 +35,9 @@ from mpdaf.drs import PixTable
 from astropy.io import fits as pyfits
 from astropy import units as u
 
+import pymusepipe
 from pymusepipe import util_pipe as upipe
-from .config_pipe import default_wave_wcs, AO_mask_lambda
+from .config_pipe import default_wave_wcs, AO_mask_lambda, dic_extra_filters
 
 from .cube_convolve import cube_kernel, cube_convolve
 
@@ -84,29 +85,39 @@ def integrate_spectrum(spectrum, wave_filter, throughput_filter, AO_mask=False):
     return flux_cont
 
 class MuseCubeMosaic(CubeMosaic):
-    def __init__(self, output_wcs, folder_cubes="",
+    def __init__(self, ref_wcs, folder_ref_wcs="", folder_cubes="",
                  prefix_cubes="DATACUBE_FINAL_WCS",
-                 list_suffix=[], verbose=False):
+                 list_suffix=[], use_fixed_cubes=True,
+                 prefix_fixed_cubes="tmask", verbose=False):
 
         self.verbose = verbose
         self.folder_cubes = folder_cubes
+        self.folder_ref_wcs = folder_ref_wcs
+        self.ref_wcs = ref_wcs
         if not self._check_folder():
             return
+        full_wcs_name = joinpath(self.folder_ref_wcs, self.ref_wcs)
 
         self.prefix_cubes = prefix_cubes
         self.list_suffix = list_suffix
+        self.prefix_fixed_cubes = prefix_fixed_cubes
+        self.use_fixed_cubes = use_fixed_cubes
 
         # Building the list of cubes
         self.build_list()
 
         # Initialise the super class
-        super(MuseCubeMosaic, self).__init__(self.list_cubes, output_wcs)
+        super(MuseCubeMosaic, self).__init__(self.list_cubes, full_wcs_name)
 
     def _check_folder(self):
         if not os.path.isdir(self.folder_cubes):
             upipe.print_error("Cube Folder {} does not exists \n"
                               "- Aborting".format(self.folder_cubes))
             return False
+        if not os.path.isdir(self.folder_ref_wcs):
+            upipe.print_error("WCS Folder {} does not exists \n"
+                              "- Aborting".format(self.folder_ref_wcs))
+            return False
         return True
 
     def build_list(self, folder_cubes=None, prefix_cubes=None, **kwargs):
@@ -132,6 +143,37 @@ class MuseCubeMosaic(CubeMosaic):
         # get the list of cubes and return if 0 found
         list_cubes = glob.glob("{0}{1}*.fits".format(self.folder_cubes,
                                                      self.prefix_cubes))
+        # Take (or not) the fixed pixtables
+        if self.use_fixed_cubes:
+            upipe.print_warning("Using Fixed cubes with prefix {} "
+                                "when relevant".format(self.prefix_fixed_cubes))
+            prefix_to_consider = "{0}{1}".format(self.prefix_fixed_cubes,
+                                                 self.prefix_cubes)
+            list_fixed_cubes = glob.glob("{0}{1}*fits".format(
+                                               self.folder_cubes,
+                                               prefix_to_consider))
+            upipe.print_info("Initial set of {:02d} fixed "
+                             "cubes found".format(len(list_fixed_cubes)))
+
+            # Looping over the existing fixed pixtables
+            for fixed_cube in list_fixed_cubes:
+                # Finding the name of the original one
+                orig_cube = fixed_cube.replace(prefix_to_consider,
+                                               self.prefix_cubes)
+                if orig_cube in list_cubes:
+                    # If it exists, remove it
+                    list_cubes.remove(orig_cube)
+                    # and add the fixed one
+                    list_cubes.append(fixed_cube)
+                    upipe.print_warning("Fixed Cube {} was included "
+                                        "in the list".format(fixed_cube))
+                    upipe.print_warning("and Cube {} was thus removed from "
+                                        "the list".format(orig_cube))
+                else:
+                    upipe.print_warning("Original Cube {} not "
+                                        "found".format(orig_cube))
+                    upipe.print_warning("Hence will not include fixed "
+                                        "Cube in the list {}".format(fixed_cube))
 
         # if the list of suffix is empty, just use all cubes
         if len(self.list_suffix) == 0:
@@ -156,7 +198,8 @@ class MuseCubeMosaic(CubeMosaic):
             upipe.print_info("Found {} cubes in this folder".format(
                                  self.ncubes))
 
-    def madcombine(self, folder_cubes=None, outcube_name="dummy.fits", mad=True):
+    def madcombine(self, folder_cubes=None, outcube_name="dummy.fits",
+                   fakemode=False, mad=True):
         """Combine the CubeMosaic and write it out.
 
         Args:
@@ -171,7 +214,8 @@ class MuseCubeMosaic(CubeMosaic):
         # Combine
         upipe.print_info("Starting the combine of "
                          "all {} input cubes".format(self.ncubes))
-        cube, expmap, statpix, rejmap = self.pycombine(mad=mad)
+        if not fakemode:
+            cube, expmap, statpix, rejmap = self.pycombine(mad=mad)
 
         # Saving
         if folder_cubes is not None:
@@ -180,8 +224,10 @@ class MuseCubeMosaic(CubeMosaic):
                 return
 
         full_cube_name = joinpath(self.folder_cubes, outcube_name)
-        upipe.print_info("Writing the new Cube {}".format(full_cube_name))
-        cube.write(full_cube_name)
+        self.mosaic_cube_name = full_cube_name
+        if not fakemode:
+            upipe.print_info("Writing the new Cube {}".format(full_cube_name))
+            cube.write(full_cube_name)
 
 class MuseCube(Cube):
     """Wrapper around the mpdaf Cube functionalities
@@ -264,7 +310,7 @@ class MuseCube(Cube):
         subcube.write(joinpath(cube_folder, outcube_name))
         return cube_folder, outcube_name
 
-    def astropy_convolve(self, other, fft=True, perslice=True, inplace=False):
+    def astropy_convolve(self, other, fft=True, inplace=False):
         """Convolve a DataArray with an array of the same number of dimensions
         using a specified convolution function.
 
@@ -354,15 +400,14 @@ class MuseCube(Cube):
 
         # Calling the external function now
         out._data, out._var = cube_convolve(out._data, kernel,
-                                            variance=out._var,
-                                            fft=fft, perslice=perslice)
+                                            variance=out._var, fft=fft)
 
         return out
 
     def convolve_cube_to_psf(self, target_fwhm, target_nmoffat=None,
                              input_function='moffat', target_function="gaussian",
                              outcube_name=None, factor_fwhm=3,
-                             fft=True, perslice=True):
+                             fft=True):
         """Convolve the cube for a target function 'gaussian' or 'moffat'
 
         Args:
@@ -378,7 +423,7 @@ class MuseCube(Cube):
                 more efficient as the convolution is done via mpdaf directly.
 
         Creates:
-            Convolved cube
+            Folder and convolved cube names
         """
         # Separate folder and name of file
         cube_folder, cube_name = os.path.split(self.filename)
@@ -405,8 +450,7 @@ class MuseCube(Cube):
                                scale=scale_spaxel, compute_kernel='pypher')
 
         # Calling the local method using astropy convolution
-        conv_cube = self.astropy_convolve(other=kernel3d, fft=fft,
-                                          perslice=perslice)
+        conv_cube = self.astropy_convolve(other=kernel3d, fft=fft)
 
         # Write the output
         upipe.print_info("Writing up the derived cube")
@@ -434,6 +478,8 @@ class MuseCube(Cube):
             **kwargs:
 
         Returns:
+            cube_folder, outcube_name (str, str): the name of the folder where
+               the output cube is, and its name
 
         """
 
@@ -524,24 +570,39 @@ class MuseCube(Cube):
                 title="{0} map".format(line))
 
     def get_filter_image(self, filter_name=None, own_filter_file=None, filter_folder="",
-            dic_extra_filters=None):
+            dic_filters=None):
         """Get an image given by a filter. If the filter belongs to
         the filter list, then use that, otherwise use the given file
         """
         try:
-            upipe.print_info("Reading MUSE filter {0}".format(filter_name))
-            refimage = self.get_band_image(filter_name)
+            upipe.print_info("Building Image with MUSE "
+                             "filter {0}".format(filter_name))
+            if filter_name.lower() == "white":
+                # filter_wave = np.array([4000.,5000.,6000.,7000.,
+                #                         8000.,9000.,10000.])
+                # filter_sensitivity = np.ones_like(filter_wave)
+                # refimage = self.bandpass_image(filter_wave, filter_sensitivity,
+                #                            interpolation='linear')
+                refimage = self.sum(axis=0)
+            else:
+                refimage = self.get_band_image(filter_name)
         except ValueError:
             # initialise the filter file
             upipe.print_info("Reading private reference filter {0}".format(filter_name))
-            if dic_extra_filters is not None:
-                if filter_name in dic_extra_filters:
-                    filter_file = dic_extra_filters[filter_name]
-                else:
-                    upipe.print_error("[mpdaf_pipe / get_filter_image] "
-                                      "Filter name not in private dictionary - Aborting")
-                    return
+
+            # First we check the extra dictionary if provided
+            if dic_filters is not None:
+                if filter_name in dic_filters:
+                    filter_file = dic_filters[filter_name]
+            # then we check the package internal filter list
+            elif filter_name in dic_extra_filters:
+                upipe.print_info("Found filter in pymusepipe internal dictionary "
+                                 "(see data/Filters)")
+                filter_folder = pymusepipe.__path__[0]
+                filter_file = dic_extra_filters[filter_name]
             else:
+                upipe.print_warning("[mpdaf_pipe / get_filter_image] "
+                                  "Filter name not in private dictionary - Aborting")
                 if own_filter_file is None:
                     upipe.print_error("[mpdaf_pipe / get_filter_image] "
                                       "No extra filter dictionary and "
@@ -552,7 +613,8 @@ class MuseCube(Cube):
             # Now reading the filter data
             path_filter = joinpath(filter_folder, filter_file)
             filter_wave, filter_sensitivity = np.loadtxt(path_filter, unpack=True)
-            refimage = self.bandpass_image(filter_wave, filter_sensitivity, 
+            upipe.print_info("Building Image with filter {}".format(filter_name))
+            refimage = self.bandpass_image(filter_wave, filter_sensitivity,
                                            interpolation='linear')
             key = 'HIERARCH ESO DRS MUSE FILTER NAME'
             refimage.primary_header[key] = (filter_name, 'filter name used')
@@ -562,6 +624,77 @@ class MuseCube(Cube):
 
         return refimage
 
+    def mask_trail(self, pq1=[0, 0], pq2=[10, 10], width=1.0,
+                   margins=0., reset=False, save=True, **kwargs):
+        """Build a cube mask from 2 points measured from a trail on an image
+
+        Input
+        -----
+        pq1: array or tuple (float)
+            p and q coordinates of point 1 along the trail
+        pq2: array or tuple (float)
+            p and q coordinates of point 2 along the trail
+        width: float
+            Value (in pixel) of the full slit width to exclude
+        margins: float
+            Value (in pixel) to extend the slit beyond the 2 extrema
+            If 0, this means limiting it to the extrema themselves.
+            Default is None, which mean infinitely long slit
+        reset (bool): if True, reset the mask before masking the slit
+        save (bool): if True, save the masked cube
+        """
+        # If width = 0 just don't do anything
+        if width <= 0.:
+            upipe.print_warning("Trail width is 0, hence no doing anything")
+            return
+        w2 = width / 2.
+
+        # Calculating the coordinates of the p and q's
+        # X2 - X1 -> note that X is q, hence the second value
+        # Y2 - Y1 -> note that Y is p, hence the first value
+        pq1 = np.asarray(pq1)
+        pq2 = np.asarray(pq2)
+        vect = pq2 - pq1
+        nvect = vect / np.hypot(vect[0], vect[1])
+        ovect = np.array([nvect[1], -nvect[0]])
+
+        # New corners depending on width and margins
+        corner1 = pq1 - margins * nvect + w2 * ovect
+        corner2 = pq1 - margins * nvect - w2 * ovect
+        corner3 = pq2 + margins * nvect - w2 * ovect
+        corner4 = pq2 + margins * nvect + w2 * ovect
+        # This is the new tuple to feed mask_polygon
+        pol = [tuple(corner1), tuple(corner2), tuple(corner3), tuple(corner4)]
+
+        out = self.copy()
+        # Reset mask if requested
+        if reset:
+            out.unmask()
+
+        out.mask_polygon(pol, unit_poly=None, inside=True)
+
+        # Rewrite a new cube
+        if save:
+            suffix_out = kwargs.pop("suffix_out", "tmask")
+            use_folder = kwargs.pop("use_folder", True)
+
+            cube_folder, cube_name = os.path.split(self.filename)
+            trailcube_name = "{0}{1}".format(suffix_out, cube_name)
+            if use_folder:
+                trailcube_name = joinpath(cube_folder, trailcube_name)
+
+            upipe.print_info("Writing the new Cube {}".format(trailcube_name))
+            out.write(trailcube_name)
+
+    def save_mask(self, mask_name="dummy_mask.fits"):
+        """Save the mask into a 0-1 image
+        """
+        newimage = self.copy()
+        newimage.data = np.where(self.mask, 0, 1).astype(np.int)
+        newimage.mask[:, :] = False
+        newimage.write(mask_name)
+
+
 class MuseSkyContinuum(object):
     def __init__(self, filename):
         self.filename = filename
@@ -783,7 +916,6 @@ class MuseImage(Image):
         # Mask based on the extent of the slit
         if extent is not None:
             y01, x01 = ind[0] - pq1[0], ind[1] - pq1[1]
-            y02, x02 = ind[0] - pq2[0], ind[1] - pq2[1]
             angle21 = np.arctan2(y01, x01) - np.arctan2(y21, x21)
             xcoord1 = np.hypot(x01, y01) * np.cos(angle21)
             ext_mask = (xcoord1 > -extent) & (xcoord1 - np.hypot(x21, y21) < extent)  
