@@ -15,14 +15,19 @@ from os.path import join as joinpath
 import numpy as np
 
 from pymusepipe import util_pipe as upipe
-from pymusepipe.musepipe import MusePipe
-from pymusepipe.config_pipe import (PHANGS_reduc_config,
-                                    default_short_PHANGS_filter_list,
-                                    default_short_filter_list)
-from pymusepipe.init_musepipe import InitMuseParameters
-from pymusepipe.combine import MusePointings
-from pymusepipe.align_pipe import rotate_pixtables
-from .prep_recipes_pipe import dic_products_scipost
+from .musepipe import MusePipe
+from .config_pipe import (PHANGS_reduc_config,
+                          default_short_PHANGS_filter_list,
+                          default_PHANGS_filter_list,
+                          default_short_filter_list,
+                          default_filter_list,
+                          default_prefix_wcs,
+                          default_prefix_wcs_mosaic)
+from .init_musepipe import InitMuseParameters
+from .combine import MusePointings
+from .align_pipe import rotate_pixtables
+from .mpdaf_pipe import MuseCubeMosaic, MuseCube
+from .prep_recipes_pipe import dict_products_scipost
 from .version import __version__ as version_pack
 
 from astropy.table import Table
@@ -30,7 +35,7 @@ from astropy.table import Table
 # ----------------- Galaxies and Pointings ----------------#
 # Sample of galaxies
 # For each galaxy, we provide the pointings numbers and the run attached to that pointing
-dic_SAMPLE_example = {
+dict_SAMPLE_example = {
         "NGC628": ['P100', {1:0, 2:0, 3:0, 4:0, 5:0, 6:0, 7:0, 8:0, 9:0, 10:0, 11:0, 12:0}],
         "NGC1087": ['P101', {1:1}], 
         }
@@ -170,7 +175,7 @@ class MusePipeSample(object):
                 - The second one is the list of pointings, itself a dictionary
                   with a 0 or 1 for each pointing number depending on whether
                   this should be included in the reduction or not.
-                  Results can be seen in self.dic_targets dictionary.
+                  Results can be seen in self.dict_targets dictionary.
         rc_filename: str
             Default to None
         cal_filename: str
@@ -198,10 +203,21 @@ class MusePipeSample(object):
 
         # Initialisation of rc and cal files
         self._init_calib_files()
+        # Setting the filters
+        if self.__phangs:
+            self._short_filter_list = kwargs.pop("short_filter_list",
+                                                 default_short_PHANGS_filter_list)
+            self._filter_list = kwargs.pop("filter_list",
+                                           default_PHANGS_filter_list)
+        else:
+            self._short_filter_list = kwargs.pop("short_filter_list",
+                                                 default_short_filter_list)
+            self._filter_list = kwargs.pop("filter_list",
+                                           default_filter_list)
 
         # Initialisation of targets
         self.init_pipes = kwargs.pop("init_pipes", True)
-        self. add_targetname = kwargs.pop("add_targetname", True)
+        self.add_targetname = kwargs.pop("add_targetname", True)
         self._init_targets()
 
     def _init_calib_files(self):
@@ -218,6 +234,7 @@ class MusePipeSample(object):
         self.root_path = init_cal_params.root
         self._subfolders = np.unique([self.sample[targetname][0]
                                 for targetname in self.targetnames])
+
         for subfolder in self._subfolders:
             update_calib_file(rc_filename, subfolder, folder_config=folder_config)
             update_calib_file(cal_filename, subfolder, folder_config=folder_config)
@@ -303,13 +320,13 @@ class MusePipeSample(object):
                                              rc_filename=rc_filename,
                                              cal_filename=cal_filename,
                                              folder_config=folder_config,
-                                             verbose=self.verbose)
+                                             verbose=False)
             self.targets[targetname].combcubes_path = init_comb_target.paths.cubes
 
             if self.init_pipes:
                 self.set_pipe_target(targetname)
 
-    def _check_pointings(self, targetname, list_pointings):
+    def _check_pointings_list(self, targetname, list_pointings):
         """Check if pointing is in the list of pointings
         Returns the list of pointings if ok. If not, return an empty list
 
@@ -327,15 +344,20 @@ class MusePipeSample(object):
         """
         # Info of the pointings and extracting the observing run for each pointing
         target_pointings = self.targets[targetname].list_pointings
-        # if no list_pointings we just do them all
-        if list_pointings == None:
-            list_pointings = target_pointings
+
+        # Now the list of pointings
+        if list_pointings is None:
+            return target_pointings
         else:
-            if any([_ not in target_pointings for _ in list_pointings]) :
-                upipe.print_error("ERROR: no pointing {0} for the given target".format(
-                                    list_pointings))
-                return []
-        return list_pointings
+            checked_pointings_list = []
+            # Check they exist
+            for pointing in list_pointings:
+                if pointing not in target_pointings:
+                    upipe.print_warning("No pointing {} for the given "
+                                        "target".format(pointing))
+                else:
+                    checked_pointings_list.append(pointing)
+            return checked_pointings_list
 
     def _check_targetname(self, targetname):
         """Check if targetname is in list
@@ -382,7 +404,7 @@ class MusePipeSample(object):
         upipe.print_info("=== Initialising MusePipe for Target {name} ===".format(name=targetname))
 
         # Check if pointings are valid
-        list_pointings = self._check_pointings(targetname, list_pointings)
+        list_pointings = self._check_pointings_list(targetname, list_pointings)
         if len(list_pointings) == 0:
             return
 
@@ -523,6 +545,86 @@ class MusePipeSample(object):
         self.reduce_target(targetname=targetname, list_pointings=list_pointings, 
                 first_recipe="align_bypointing", **kwargs)
 
+    def finalise_reduction(self, targetname=None, rot_pixtab=False, create_wcs=True,
+                           create_expocubes=True, offset_table_name=None,
+                           folder_offset_table=None,
+                           dict_exposures=None,
+                           **kwargs):
+        """Finalise the reduction steps by using the offset table, rotating the
+        pixeltables, then reconstructing the PIXTABLE_REDUCED, produce reference
+        WCS for each pointing, and then run the reconstruction of the final
+        individual cubes
+
+        Args:
+            targetname (str):
+            rot_pixtab (bool):
+            create_wcs (bool):
+            create_expocubes (bool):
+            offset_table_name (str):
+            folder_offset_table (str):
+            **kwargs:
+
+        Returns:
+
+        """
+        if rot_pixtab:
+            # We first use the offset table to rotate the pixtables
+            upipe.print_info("========== ROTATION OF PIXTABLES ===============")
+            self.rotate_pixtables_target(targetname=targetname,
+                                         folder_offset_table=folder_offset_table,
+                                         offset_table_name=offset_table_name,
+                                         fakemode=False)
+            # We then reconstruct the pixtable reduced so we can
+            # redo a muse_exp_combine if needed
+            upipe.print_info("==== REDUCED PIXTABLES for REFERENCE MOSAIC ====")
+            self.run_target_scipost_perexpo(targetname=targetname,
+                                            folder_offset_table=folder_offset_table,
+                                            offset_table_name=offset_table_name,
+                                            save="individual",
+                                            wcs_auto=False,
+                                            norm_skycontinuum=True,
+                                            dict_exposures=dict_exposures)
+
+        if create_wcs:
+            # Creating the WCS reference frames. Full mosaic and individual
+            # Pointings.
+            upipe.print_info("=========== CREATION OF WCS MASKS ==============")
+            dict_exposures = kwargs.pop("dict_exposures", None)
+            mosaic_wcs = kwargs.pop("mosaic_wcs", True)
+            reference_cube = kwargs.pop("reference_cube", True)
+            pointings_wcs = kwargs.pop("pointings_wcs", True)
+            list_pointings = kwargs.get("list_pointings", None)
+            self.create_reference_wcs(targetname=targetname,
+                                      folder_offset_table=folder_offset_table,
+                                      offset_table_name=offset_table_name,
+                                      dict_exposures=dict_exposures,
+                                      reference_cube=reference_cube,
+                                      mosaic_wcs=mosaic_wcs,
+                                      pointings_wcs=pointings_wcs,
+                                      list_pointings=list_pointings,
+                                      fakemode=False)
+
+        if create_expocubes:
+            # Running the individual cubes now with the same WCS reference
+            upipe.print_info("=========== CREATION OF EXPO CUBES =============")
+            self.run_target_scipost_perexpo(targetname=targetname,
+                                            folder_offset_table=folder_offset_table,
+                                            offset_table_name=offset_table_name,
+                                            save="cube",
+                                            norm_skycontinuum=True,
+                                            wcs_auto=True,
+                                            dict_exposures=dict_exposures,
+                                            **kwargs)
+
+        if create_pointingcubes:
+            # Running the pointing cubes now with the same WCS reference
+            upipe.print_info("========= CREATION OF POINTING CUBES ===========")
+            self.combine_target_per_pointing(targetname=targetname,
+                                             offset_table_name=final_offset_table,
+                                             folder_offset_table=folder_offset_table,
+                                             dict_exposures=dict_exposures,
+                                             filter_list=self._short_filter_list)
+
     def run_target_scipost_perexpo(self, targetname=None, list_pointings=None,
                                    folder_offset_table=None, offset_table_name=None,
                                    **kwargs):
@@ -537,43 +639,39 @@ class MusePipeSample(object):
 
         """
         # Check if pointings are valid
-        list_pointings = self._check_pointings(targetname, list_pointings)
+        list_pointings = self._check_pointings_list(targetname, list_pointings)
         if len(list_pointings) == 0:
             return
 
         # WCS imposed by setting the reference
         add_targetname = kwargs.pop("add_targetname", self.add_targetname)
         prefix_all = kwargs.pop("prefix_all", "")
+        cube_suffix = dict_products_scipost['cube'][0]
         if add_targetname:
             prefix_all = "{0}_{1}".format(targetname, prefix_all)
+            cube_suffix = "{0}_{1}".format(targetname, cube_suffix)
         save = kwargs.pop("save", 'cube')
 
         wcs_auto = kwargs.pop("wcs_auto", True)
+        wcs_suffix = "{0}{1}".format(default_prefix_wcs, cube_suffix)
         if not wcs_auto:
             name_wcs = kwargs.pop("name_wcs", None)
 
         # Fetch the default folder for the WCS files which is the folder
         # of the Combined cubes
-#        self.init_combine(targetname=targetname, folder_offset_table=folder_offset_table,
-#                          offset_table_name=offset_table_name)
-#        default_comb_folder = self.pipes_combine[targetname].paths.cubes
         default_comb_folder = self.targets[targetname].combcubes_path
         # Now fetch the value set by the user
         folder_ref_wcs = kwargs.pop("folder_ref_wcs", default_comb_folder)
-        if self.__phangs:
-            filter_list = kwargs.pop("filter_list", default_short_PHANGS_filter_list)
-        else:
-            filter_list = kwargs.pop("filter_list", default_short_filter_list)
+        filter_list = kwargs.pop("filter_list", self._short_filter_list)
 
         # Running the scipost_perexpo for all pointings individually
         for pointing in list_pointings:
             if wcs_auto:
-                cube_suffix = dic_products_scipost['cube'][0]
-                if add_targetname:
-                     cube_suffix = "{0}_{1}".format(targetname, cube_suffix)
-                name_wcs = "{0}_P{1:02d}.fits".format(cube_suffix,
-                                                      np.int(pointing))
-            suffix = "_WCS_P{0:02d}".format(np.int(pointing))
+                name_wcs = "{0}_P{1:02d}.fits".format(wcs_suffix, np.int(pointing))
+            if name_wcs is not None:
+                suffix = "_WCS_P{0:02d}".format(np.int(pointing))
+            else:
+                suffix = "_P{0:02d}".format(np.int(pointing))
             kwargs_pointing = {'ref_wcs': name_wcs,
                                'suffix': suffix,
                                'folder_ref_wcs': folder_ref_wcs,
@@ -611,34 +709,38 @@ class MusePipeSample(object):
             _ = kwargs.pop(key, default)
             kwargs_recipe[key] = default
 
-        # some parameters which depend on the pointings for this recipe
-        kwargs_per_pointing = kwargs.pop("kwargs_per_pointing", {})
-
         # Initialise the pipe if needed
         self.set_pipe_target(targetname=targetname, list_pointings=list_pointings,
                 first_recipe=recipe_name, last_recipe=recipe_name, **kwargs)
 
         # Check if pointings are valid
-        list_pointings = self._check_pointings(targetname, list_pointings)
+        list_pointings = self._check_pointings_list(targetname, list_pointings)
         if len(list_pointings) == 0:
             return
 
+        # some parameters which depend on the pointings for this recipe
+        kwargs_per_pointing = kwargs.pop("kwargs_per_pointing", {})
+        param_recipes = kwargs.pop("param_recipes", {})
+
         # Loop on the pointings
         for pointing in list_pointings:
             upipe.print_info("====== START - POINTING {0:2d} "
                              "======".format(pointing))
-            param_recipes = {}
+
+            this_param_recipes = copy.deepcopy(param_recipes)
             if pointing in kwargs_per_pointing:
-                param_recipes[recipe_name] = kwargs_per_pointing[pointing]
+                if recipe_name in kwargs_per_pointing[pointing]:
+                    this_param_recipes[recipe_name].update(
+                                     kwargs_per_pointing[pointing][recipe_name])
 
             # Initialise raw tables if not already done (takes some time)
             if not self.pipes[targetname][pointing]._raw_table_initialised:
                 self.pipes[targetname][pointing].init_raw_table(overwrite=True)
             if self.__phangs:
-                self.pipes[targetname][pointing].run_phangs_recipes(param_recipes=param_recipes,
+                self.pipes[targetname][pointing].run_phangs_recipes(param_recipes=_this_param_recipes,
                                                                     **kwargs_recipe)
             else:
-                self.pipes[targetname][pointing].run_recipes(param_recipes=param_recipes,
+                self.pipes[targetname][pointing].run_recipes(param_recipes=this_param_recipes,
                                                              **kwargs_recipe)
             upipe.print_info("====== END   - POINTING {0:2d} ======".format(pointing))
 
@@ -660,9 +762,12 @@ class MusePipeSample(object):
         upipe.print_info("---- Starting the Data Reduction for Target={0} ----".format(
                             targetname))
 
+        # Get the parameters for the recipes
+        param_recipes = kwargs.pop("param_recipes", {})
         kwargs_recipe = {}
-        for key, default in zip(['fraction', 'skymethod', 'illum'], [0.8, "model", True]):
-            item = kwargs.pop(key, default)
+        for key, default in zip(['fraction', 'skymethod', 'illum'],
+                                [0.8, "model", True]):
+            _ = kwargs.pop(key, default)
             kwargs_recipe[key] = default
 
         # Initialise the pipe if needed
@@ -671,7 +776,7 @@ class MusePipeSample(object):
             self.set_pipe_target(targetname=targetname, list_pointings=list_pointings, **kwargs)
 
         # Check if pointings are valid
-        list_pointings = self._check_pointings(targetname, list_pointings)
+        list_pointings = self._check_pointings_list(targetname, list_pointings)
         if len(list_pointings) == 0:
             return
 
@@ -682,9 +787,11 @@ class MusePipeSample(object):
             if not self.pipes[targetname][pointing]._raw_table_initialised:
                 self.pipes[targetname][pointing].init_raw_table(overwrite=True)
             if self.__phangs:
-                self.pipes[targetname][pointing].run_phangs_recipes(**kwargs_recipe)
+                self.pipes[targetname][pointing].run_phangs_recipes(param_recipes=param_recipes,
+                                                                    **kwargs_recipe)
             else:
-                self.pipes[targetname][pointing].run_recipes(**kwargs_recipe)
+                self.pipes[targetname][pointing].run_recipes(param_recipes=param_recipes,
+                                                             **kwargs_recipe)
             upipe.print_info("====== END   - POINTING {0:2d} ======".format(pointing))
 
     def rotate_pixtables_target(self, targetname=None, list_pointings=None,
@@ -703,14 +810,11 @@ class MusePipeSample(object):
                                  list_pointings=list_pointings, **kwargs)
 
         # Check if pointings are valid
-        list_pointings = self._check_pointings(targetname, list_pointings)
+        list_pointings = self._check_pointings_list(targetname, list_pointings)
         if len(list_pointings) == 0:
             return
 
-        # add_targetname = kwargs.pop("add_targetname", self.add_targetname)
         prefix = kwargs.pop("prefix", "")
-        # if add_targetname:
-        #     prefix = "{}_{}".format(targetname, prefix)
         if folder_offset_table is None:
             folder_offset_table = self.pipes[targetname][list_pointings[0]].paths.alignment
         offset_table = Table.read(joinpath(folder_offset_table, offset_table_name))
@@ -731,18 +835,19 @@ class MusePipeSample(object):
                              list_ifu=None, angle=angle, fakemode=fakemode,
                              prefix=prefix, **kwargs)
 
-    def init_mosaic(self, targetname=None, list_pointings="all", **kwargs):
+    def init_mosaic(self, targetname=None, list_pointings=None, **kwargs):
         """Prepare the combination of targets
 
         Input
         -----
         targetname: str [None]
             Name of target
-        list_pointings: list [or "all"=default]
+        list_pointings: list [or None=default meaning all pointings]
             List of pointings (e.g., [1,2,3])
         """
+        add_targetname = kwargs.pop("add_targetname", self.add_targetname)
         # Check if pointings are valid
-        list_pointings = self._check_pointings(targetname, list_pointings)
+        list_pointings = self._check_pointings_list(targetname, list_pointings)
         if len(list_pointings) == 0:
             return
 
@@ -750,34 +855,88 @@ class MusePipeSample(object):
         list_pointing_names = ["P{0:02d}".format(np.int(pointing))
                                for pointing in list_pointings]
 
-        folder_ref_wcs = kwargs.pop("folder_ref_wcs", comb_folder)
-        self.pipes_mosaic[targetname] = MuseCubeMosaic(output_wcs=...,
-                                                       folder_cubes=...,
-                                                       prefix_cubes="DATACUBE_FINAL_WCS",
-                                                       list_suffix=list_pointing_names)
+        default_comb_folder = self.targets[targetname].combcubes_path
+        folder_ref_wcs = kwargs.pop("folder_ref_wcs", default_comb_folder)
+        folder_cubes = kwargs.pop("folder_cubes", default_comb_folder)
+        prefix_cubes = "DATACUBE_FINAL_WCS"
+        if add_targetname:
+            wcs_prefix = "{}_".format(targetname)
+            prefix_cubes = "{0}_{1}".format(targetname, prefix_cubes)
+        else:
+            wcs_prefix = ""
+        ref_wcs = kwargs.pop("ref_wcs", "{0}{1}DATACUBE_FINAL.fits".format(
+                                 default_prefix_wcs_mosaic, wcs_prefix))
+        self.pipes_mosaic[targetname] = MuseCubeMosaic(ref_wcs=ref_wcs,
+                                                       folder_ref_wcs=folder_ref_wcs,
+                                                       folder_cubes=folder_cubes,
+                                                       prefix_cubes=prefix_cubes,
+                                                       list_suffix=list_pointing_names,
+                                                       **kwargs)
 
-    def mosaic(self, targetname=None, list_pointings="all", **kwargs):
+    def mosaic(self, targetname=None, list_pointings=None, **kwargs):
+        """
 
+        Args:
+            targetname:
+            list_pointings:
+            **kwargs:
+
+        Returns:
+
+        """
+
+        add_targetname = kwargs.pop("add_targetname", self.add_targetname)
+        build_cube = kwargs.pop("build_cube", True)
+        build_images = kwargs.pop("build_images", True)
+        dict_expo = kwargs.pop("dict_exposures", None)
         self.init_mosaic(targetname=targetname, list_pointings=list_pointings,
-                         **kwargs)
+                         add_targetname=add_targetname,
+                         dict_exposures=dict_expo)
 
         # Doing the mosaic with mad
-        self.pipes_mosaic[targetname].madcombine()
-        # Constructing the images for that mosaic
-        if self.__phangs:
-            filter_list = kwargs.pop("filter_list", default_PHANGS_filter_list)
-        else:
-            filter_list = kwargs.pop("filter_list", default_filter_list)
+        suffix = kwargs.pop("suffix", "WCS_Pall_mad")
+        default_comb_folder = self.targets[targetname].combcubes_path
+        folder_cubes = kwargs.pop("folder_cubes", default_comb_folder)
 
-    def init_combine(self, targetname=None, list_pointings="all",
-                     folder_offset_table=None, offset_table_name=None, **kwargs):
+        default_cube_name = "{0}_DATACUBE_FINAL_{1}.fits".format(targetname, suffix)
+        default_cube_name = joinpath(folder_cubes, default_cube_name)
+
+
+        outcube_name = kwargs.pop("output_cube_name", default_cube_name)
+        outcube_name = joinpath(folder_cubes, outcube_name)
+        # Doing the MAD combination using mpdaf. Note the build_cube fakemode
+        self.pipes_mosaic[targetname].madcombine(outcube_name=outcube_name,
+                                                 fakemode=not build_cube)
+
+        if build_images:
+            # Constructing the images for that mosaic
+            filter_list = (kwargs.pop("filter_list",
+                                      self._filter_list)).split(',')
+
+            mosaic_name = self.pipes_mosaic[targetname].mosaic_cube_name
+            if not os.path.isfile(mosaic_name):
+                upipe.print_error("Mosaic cube file does not exist = {} \n"
+                                  "Aborting Image reconstruction".format(mosaic_name))
+                return
+            cube = MuseCube(filename=mosaic_name)
+            upipe.print_info("Building images for each filter in the list")
+            for filter in filter_list:
+                upipe.print_info("Filter = {}".format(filter))
+                ima = cube.get_filter_image(filter_name=filter)
+                ima_name = "{0}_IMAGE_FOV_{1}_{2}.fits".format(targetname, filter,
+                                                               suffix)
+                ima.write(joinpath(folder_cubes, ima_name))
+
+    def init_combine(self, targetname=None, list_pointings=None,
+                     folder_offset_table=None, offset_table_name=None,
+                     **kwargs):
         """Prepare the combination of targets
 
         Input
         -----
         targetname: str [None]
             Name of target
-        list_pointings: list [or "all"=default]
+        list_pointings: list [or None=default= all pointings]
             List of pointings (e.g., [1,2,3])
         offset_table_name: str
             Name of Offset table
@@ -792,19 +951,25 @@ class MusePipeSample(object):
                                                  folder_offset_table=folder_offset_table,
                                                  log_filename=log_filename, **kwargs)
 
+    def combine_target_per_pointing(self, targetname=None, **kwargs):
+        """Run the combine recipe. Shortcut for combine[targetname].run_combine()
+        """
+        self.init_combine(targetname=targetname, **kwargs)
+        self.pipes_combine[targetname].run_combine_all_single_pointings(**kwargs)
+
     def combine_target(self, targetname=None, **kwargs):
         """Run the combine recipe. Shortcut for combine[targetname].run_combine()
         """
         self.init_combine(targetname=targetname, **kwargs)
-        self.pipes_combine[targetname].run_combine()
+        self.pipes_combine[targetname].run_combine(**kwargs)
 
-    def combine_target_pointings_withmasks(self, targetname=None, 
-            combine=True, masks=True, perpointing_combine=True, mosaic_wcs=True,
-            **kwargs):
+    def create_reference_wcs(self, targetname=None, pointings_wcs=True,
+                             mosaic_wcs=True, reference_cube=True, **kwargs):
         """Run the combine for individual exposures first building up
         a mask.
         """
         self.init_combine(targetname=targetname, **kwargs)
-        self.pipes_combine[targetname].run_combine_all_single_pointings_withmasks(
-            combine=combine, masks=masks, perpointing_combine=perpointing_combine,
-            mosaic_wcs=mosaic_wcs, **kwargs)
+        self.pipes_combine[targetname].create_reference_wcs(pointings_wcs=pointings_wcs,
+                                                  mosaic_wcs=mosaic_wcs,
+                                                  reference_cube=reference_cube,
+                                                            **kwargs)
